//AUGSUSP: A program for studying the Augmented Suspension property for (random) graphs

/*Jason Behrstock, Mark Hagen, Tim Susse*/

#include <vector>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <iostream>
#include <fstream>
#include <string>
#include<sstream>
#include <cstring>
#include <vector>
#include <utility>

using namespace std;
using
     std::vector;

//This algorithm operates on a matrix Adj, which is the adjacency matrix of a graph G.
vector < vector < char > > Adj;

//AugSusp will be the vector that keeps track of which vertices are in the largest AS subgraph which has a given pair of disjoint vertices as "ends".

vector < int > Augsusp;

vector < int > Suspension ( int j, int k, int n );

//The Matrix Neigh stores, for each vertex i,  all vertices adjacent to i.

vector < vector < long > > Neigh;

//The functions used are as follows:

//ErdosRenyi is used to produce the adjacency matrix Adj: beginning with a set of n vertices, 
//ErdosRenyi decides independently with probability p whether each pair of vertices is adjacent.

void ErdosRenyi ( float p, int n, float r, int flag_debug, int flag_verbose );

//void ErdosRenyi(float p, int n, float r);

//convert line of input to teaformat.
void convertline ( string inputline, int &n, int flag_verbose, int flag_debug, int flag_full );

//Connected checks if the graph is connected

bool Connected ( int n );

// Neighbors populates Neigh

void Neighbors ( int n );

int PairTest ( int j, int k, int n, int flag_debug, int flag_verbose, int flag_silent, int flag_full, vector < int >suspvect );

int core = 0;			//needed for silent mode


//Connected to make sure that the core is not a clique 0=non-clique 1=clique

bool Clique ( int j, int k, int n, vector < int >suspvect );

//*****************************************************************************************************
//*************************************   MAIN   ******************************************************
//*****************************************************************************************************

int main ( int argc, char *argv[] )
{
//The arguments entered on the command line are a(=p), n, and tests. The option -v activates the verbose mode. The option -f activates the full output mode (this is the same as -s, but the output is to standard out). The option -d activates the debug mode. The option -s activates silent mode.  These must be entered as the first argument.

     int
	  flag_verbose = 0, flag_full = 0, flag_debug = 0, flag_silent =
	  0, flag_counter = 0;

     string
	  out_file;

     if ( argv[1][0] == '-' )
	 {
	      if ( argv[1][1] == 'v' )
		  {
		       flag_verbose = 1;
		       flag_counter = 1;
		  }
	      if ( argv[1][2] == 'v' )
		  {
		       flag_verbose = 1;
		       flag_counter = 1;
		  }
	      if ( argv[1][3] == 'v' )
		  {
		       flag_verbose = 1;
		       flag_counter = 1;
		  }
	 }


     if ( argv[1][0] == '-' )
	 {
	      if ( argv[1][1] == 'f' )
		  {
		       flag_full = 1;
		       flag_counter = 1;
		  }
	      if ( argv[1][2] == 'f' )
		  {
		       flag_full = 1;
		       flag_counter = 1;
		  }
	      if ( argv[1][3] == 'f' )
		  {
		       flag_full = 1;
		       flag_counter = 1;
		  }
	 }

     if ( argv[1][0] == '-' )
	 {
	      if ( argv[1][1] == 'd' )
		  {
		       flag_debug = 1;
		       flag_counter = 1;
		  }
	      if ( argv[1][2] == 'd' )
		  {
		       flag_debug = 1;
		       flag_counter = 1;
		  }
	      if ( argv[1][3] == 'd' )
		  {
		       flag_debug = 1;
		       flag_counter = 1;
		  }
	 }

     if ( argv[1][0] == '-' )
	 {
	      if ( argv[1][1] == 's' )
		  {
		       flag_silent = 1;
		       flag_counter = 1;
		  }
	      if ( argv[1][2] == 's' )
		  {
		       flag_silent = 1;
		       flag_counter = 1;
		  }
	      if ( argv[1][3] == 's' )
		  {
		       flag_silent = 1;
		       flag_counter = 1;
		  }
	 }

     if ( flag_silent == 1 )
	 {
	      long
		   curr = time ( NULL );

	      stringstream
		   file_name;

	      file_name << "silent_data/";
	      file_name << "silent_";
	      file_name << curr;
	      file_name << ".txt";

	      out_file = file_name.str (  );
	 }

     int
	  flag_teaformatted = 0;

     if ( argv[1][0] == '-' )
	 {
	      if ( argv[1][1] == 't' )
		  {
		       flag_teaformatted = 1;
		       flag_counter = 1;
		  }
	      if ( argv[1][2] == 't' )
		  {
		       flag_teaformatted = 1;
		       flag_counter = 1;
		  }
	      if ( argv[1][3] == 't' )
		  {
		       flag_teaformatted = 1;
		       flag_counter = 1;
		  }
	 }

//p is the density of edges, n is the number of vertices, and tests is the number of tests to run:

     float
	  p = 1;
     int
	  n = 1;
     int
	  tests = 1;

     if ( flag_teaformatted == 0 )
	 {

	      p = atof ( argv[flag_counter + 2] );

	      n = atoi ( argv[flag_counter + 1] ), tests =
		   atoi ( argv[flag_counter + 3] );
	 }

     string
	  inputfile;

     string
	  inputfilename;

     string
	  inputline;

     // as_total will report the number of tests which find an AS graph
     int
	  as_total = 0;

     // prop_as will tell the percent of tests which are AS
     float
	  prop_as = 0;

     // r will be the random seed to produce a random graph.
     float
	  r;

     srand ( time ( NULL ) );

     if ( flag_teaformatted == 1 )
	 {
	      int
		   numLines = 0;
	      ifstream
	      inputfilename ( argv[2] );
	      string
		   unused;

	      while ( getline ( inputfilename, unused ) )
		   ++numLines;

	      tests = numLines;

	      if ( flag_full != 1 )
		  {
		       printf ( "Running %d graphs from file.\n", tests );
		  }
	 }

// This is the main loop. It will be run "test" times, as measured by the counter curr_test:

     for ( int curr_test = 0; curr_test < tests; curr_test++ )
	 {
	      int
		   increased = 0;
	      int
		   total_norm = 0;
	      int
		   total_core = 0;
	      int
		   subtotal_core = 0;

	      //First we create our graph:

	      //Random:
	      if ( flag_teaformatted == 0 )
		  {
		       //first we make the relevant vectors and resize them to n entries:

		       Adj.resize ( n );

		       for ( int i = 0; i < n; i++ )
			    Adj[i].resize ( n );

		       //create graph

		       ErdosRenyi ( p, n, r, flag_debug, flag_verbose );
		  }
	      if ( flag_teaformatted == 1 )
		  {

		       string line;

		       ifstream inputfilename ( argv[2] );

		       if ( inputfilename.is_open (  ) )
			   {
				for ( int lineno = 0;
				      getline ( inputfilename, line )
				      && lineno < tests; lineno++ )
				    {
					 if ( lineno == curr_test )
					      inputline = line;
				    }

				inputfilename.close (  );
			   }

		       convertline ( inputline, n, flag_verbose, flag_debug,
				     flag_full );

		  }

	      Augsusp.resize ( n );

//We now populate Neigh...

	      Neighbors ( n );

//...and only proceed further if the graph is connected.

	      if ( Connected ( n ) == true )
		  {
		       int
			    norm = 0;
		       int
			    tested_pair = 0;
		       int
			    tested_block = 0;

		       for ( int j = 0; j < n; j++ )
			   {
				for ( int k = 0; k < j; k++ )	
				    {
					 int
					      good_block = 0;
					 int
					      good_pair = 0;

					 //we now find the number of vertices in the block whose end is the pair of vertices j, k, together with all vertices that can be beamed up into that block.

					 vector < int >
					      suspvect;

					 if ( Adj[j][k] == 1 )
					     {
						  norm = 2;
						  if ( flag_debug == 1 )
						      {
							   printf
								( "Augsusp[]= Empty, since vertices are adjacent\n" );
						      }

						  if ( flag_verbose == 1 )
						      {
							   printf
								( "Core size: Empty\n" );
						      }

						  if ( flag_silent == 1
						       || flag_full == 1 )

						       if ( flag_verbose ==
							    1 )
							   {
								printf
								     ( "Block size + beamed-up vertices: 2\n" );
							   }

					     }

					 else
					     {
						  suspvect =
						       Suspension ( j, k, n );

						  norm =
						       PairTest ( j, k, n,
								  flag_debug,
								  flag_verbose,
								  flag_silent,
								  flag_full,
								  suspvect );
					     }



					 if ( flag_silent == 1
					      || flag_full == 1 )
					     {

						  if ( Adj[j][k] == 0 )
						      {
							   good_pair = 1;

							   if ( norm > 2 )
								good_block =
								     1;

						      }

						  total_norm =
						       total_norm +
						       good_pair * norm;
						  total_core =
						       total_core +
						       good_pair * core;
						  tested_pair =
						       tested_pair +
						       good_pair;

						  subtotal_core =
						       subtotal_core +
						       good_block * core;
						  tested_block =
						       tested_block +
						       good_block;

					     }

					 if ( norm == n )
					     {
						  if ( Clique
						       ( j, k, n,
							 suspvect ) == false )
						      {
							   as_total++;
							   increased = 1;
						      }
					     }

					 if ( increased == 1 )
					      break;
				    }

				if ( increased == 1 )
				     break;
			   }

		       if ( flag_silent == 1 )	//report on individual tests when running in silent mode:
			   {
				float
				     avg_norm_pair =
				     static_cast < float >( total_norm ) /
				     tested_pair;
				float
				     avg_core_pair =
				     static_cast < float >( total_core ) /
				     tested_pair;
				float
				     avg_norm_block =
				     static_cast < float >( total_norm ) /
				     tested_block;
				float
				     avg_core_block =
				     static_cast < float >( total_core ) /
				     tested_block;

				std::ofstream silent_output;
				silent_output.open ( out_file.c_str (  ),
						     std::ofstream::app );

				//Output (Vertices, Density, Whether AS, # good pairs before positive ID, # blocks with nonempty core before positive ID, average size of core over all good pairs, average size of core over all good blocks, average total size of \bar B over all good pairs, average total size of \bar B over all blocks):

				silent_output << n << ", " << p << ", " <<
				     increased << ", " << tested_pair << ", "
				     << tested_block << ", " << avg_core_pair
				     << ", " << avg_core_block << ", " <<
				     avg_norm_pair << ", " << avg_norm_block
				     << "\n";
				silent_output.close (  );
			   }
		       if ( flag_full == 1 )	//report on individual tests when running in silent mode:
			   {
				float
				     avg_norm_pair =
				     static_cast < float >( total_norm ) /
				     tested_pair;
				float
				     avg_core_pair =
				     static_cast < float >( total_core ) /
				     tested_pair;
				float
				     avg_norm_block =
				     static_cast < float >( total_norm ) /
				     tested_block;
				float
				     avg_core_block =
				     static_cast < float >( total_core ) /
				     tested_block;

				//Output (Vertices, Density, Whether AS, # good pairs before positive ID, # blocks with nonempty core before positive ID, average size of core over all good pairs, average size of core over all good blocks, average total size of \bar B over all good pairs, average total size of \bar B over all blocks):

				cout << n << ", " << p << ", " << increased <<
				     ", " << tested_pair << ", " <<
				     tested_block << ", " << avg_core_pair <<
				     ", " << avg_core_block << ", " <<
				     avg_norm_pair << ", " << avg_norm_block
				     << "\n";
			   }


		  }

	      else		// HAVE IT DO DIFFERENT THINGS IN THE DISCONNECTED CASE ACCORDING TO FLAGS
		  {
		       //printf("Disconnected, hence not thick.");

		       if ( flag_silent == 1 )	//report on individual tests when running in silent mode:
			   {
				std::ofstream silent_output;
				silent_output.open ( out_file.c_str (  ),
						     std::ofstream::app );

				//Output (Vertices, Density, Whether AS, # good pairs before positive ID, # blocks with nonempty core before positive ID, average size of core over all good pairs, average size of core over all good blocks, average total size of \bar B over all good pairs, average total size of \bar B over all blocks):

				silent_output << n << ", " << p << ", " << "0"
				     << ", " << "Disconnected." << "\n";
				silent_output.close (  );
			   }
		       if ( flag_full == 1 )	//report on individual tests when running in silent mode:
			   {
				//Output (Vertices, Density, Whether AS, # good pairs before positive ID, # blocks with nonempty core before positive ID, average size of core over all good pairs, average size of core over all good blocks, average total size of \bar B over all good pairs, average total size of \bar B over all blocks):

				cout << n << ", " << p << ", " << "0" << ", "
				     << "Disconnected." << "\n";
			   }
		  }
	 }

//Info to report:

     if ( flag_silent != 1 && flag_full != 1 )
	 {
	      prop_as = 100 * static_cast < float >( as_total ) /
		   tests;

	      printf ( "p=%f, n=%d, trials= %d, #AS= %d, PercentAS= %f\n", p,
		       n, tests, as_total, prop_as );
	 }

}


//***********************************************************************************************************************************
//*************************************   PairTest   **********************************************************
//***********************************************************************************************************************************

// This program builds the Augsusp vector which keeps track of which vertices are in a given "block".

int
PairTest ( int j, int k, int n, int flag_debug, int flag_verbose,
	   int flag_silent, int flag_full, vector < int >suspvect )
{
     //temp_norm will measure the size of the block

     int
	  temp_norm = 0;

     // If the verbose flag is on, then this is a counter to measure the size of each block
     int
	  just_block = 0;

     //First we build the suspension: if c is connected to j and k, then we add c to Augsusp.

     Augsusp.resize ( n );

     for ( int t = 0; t < n; t++ )
	 {
	      Augsusp[t] = 0;
	 }

     for ( int t = 0; t < suspvect.size (  ); t++ )
	 {
	      Augsusp[suspvect[t]] = 1;
	 }

     just_block = just_block + suspvect.size (  );

     if ( flag_debug == 1 )
	 {
	      for ( int c = 0; c < n; c++ )
		   printf
			( "Augsusp[%d]= %d, in first loop with j=%d and k=%d\n",
			  c, Augsusp[c], j, k );
	 }

     if ( flag_verbose == 1 )
	 {
	      printf ( "Core size: %d\n", just_block );
	 }

     if ( flag_silent == 1 || flag_full == 1 )
	 {
	      core = just_block;
	 }

// add the j and k vertex to the Augsusp, these are the two "ends".

     Augsusp[j] = 1;
     Augsusp[k] = 1;

     if ( flag_debug == 1 )
	 {
	      for ( int z = 0; z < n; z++ )
		  {
		       printf ( "Augsusp[%d]= %d\n", z, Augsusp[z] );
		  }
	 }

     for ( int a = 0; a < suspvect.size (  ); a++ )
	 {
	      for ( int b = a + 1; b < suspvect.size (  ); b++ )
		  {
		       for ( int d = 0; d < n; d++ )
			   {
//                                  printf("Before: a=%d, suspvect[a]=%d , b=%d, suspvect[b] %d, d=%d \n", a, suspvect[a], b, suspvect[b], d);

				if ( Augsusp[d] == 0
				     && Adj[suspvect[a]][suspvect[b]] == 0 )
				    {
					 Augsusp[d] =
					      Adj[suspvect[a]][d] *
					      Adj[suspvect[b]][d];

//                                  printf("After: a=%d, suspvect[a]=%d , b=%d, suspvect[b] %d, d=%d \n", a, suspvect[a], b, suspvect[b], d);

				    }
			   }
		  }
	 }

     if ( flag_debug == 1 )
	 {
	      for ( int c = 0; c < n; c++ )
		   printf
			( "Augsusp[%d]= %d, in second loop with j=%d and k=%d \n",
			  c, Augsusp[c], j, k );
	 }


     for ( int b = 0; b < n; b++ )
	  temp_norm = temp_norm + Augsusp[b];

     if ( flag_verbose == 1 )
	 {
	      printf ( "Block size + beamed-up vertices: %d\n", temp_norm );
	 }

     return temp_norm;
}

//***********************************************************************************************************************************
//********************************************   ErdosRenyi   ***********************************************************************
//***********************************************************************************************************************************

//Note: This function is adapted from a function in older software by J. Behrstock, M. Hagen, A. Sisto, available at www.wescac.net/research.html

void
ErdosRenyi ( float p, int n, float r, int flag_debug, int flag_verbose )
{
     //This function creates our random graph my creating a random symmetric matrix. It sets Adj[i][j]=Adj[j][i] to 0 or 1 according to whether a random number r in [0,1] exceeds p.

     for ( int i = 0; i < n; i++ )
	 {
	      for ( int j = i + 1; j < n; j++ )
		  {
		       r = ( float ) rand (  ) / ( float ) RAND_MAX;
		       if ( r < p )
			   {
				Adj[i][j] = 1;
				Adj[j][i] = 1;
			   }

		       else
			   {
				Adj[i][j] = 0;
				Adj[j][i] = 0;
			   }
		  }
	 }

     // This loop prints the Adj matrix (ie the random graph).
     if ( flag_debug == 1 )
	 {
	      for ( int i = 0; i < n; i++ )
		   for ( int j = 0; j < n; j++ )
			printf ( "Adj[%d][%d] = %d\n", i, j, Adj[i][j] );
	 }
     //

     //TEA formatted graphs:
     if ( flag_debug == 1 || flag_verbose == 1 )
	 {
	      printf ( "[1" );
	      for ( int i = 1; i < n; i++ )
		   printf ( ",%d", i );
	      printf ( "] [" );
	      for ( int i = 0; i < n; i++ )
		   for ( int j = i + 1; j < n; j++ )
			if ( Adj[i][j] == 1 )
			    {
				 printf ( "(%d,%d),", i, j );
			    }
	      printf ( "\b]\n" );
	 }

     //TEA formatted graphs output to file:
     std::ofstream graphsanalyzed;
     if ( flag_debug == 1 )
	 {
	      graphsanalyzed.open ( "graphsanalyzed.txt",
				    std::ofstream::app );
	      graphsanalyzed << "[1";
	      for ( int i = 1; i < n; i++ )
		   graphsanalyzed << "," << i + 1;
	      graphsanalyzed << "] [";
	      for ( int i = 0; i < n; i++ )
		   for ( int j = i + 1; j < n; j++ )
			if ( Adj[i][j] == 1 )
			    {
				 graphsanalyzed << "(" << i + 1 << "," << j +
				      1 << "),";
			    }
	      graphsanalyzed << "(1,1)]\n";
	      graphsanalyzed.close (  );
	 }
}

//***********************************************************************************************************************************
//*********************************************   Neighbors   ***********************************************************************
//***********************************************************************************************************************************

//Note: This function is adapted from a function in older software by J. Behrstock, M. Hagen, A. Sisto, available at www.wescac.net/research.html

void
Neighbors ( int n )
{
     Neigh.resize(n);
    
    for (unsigned i=0; i<Neigh.size(); i++)
    {
        Neigh[i].clear();
    }

     for ( int i = 0; i < n; i++ )
	 {
	      for ( int j = 0; j < n; j++ )
		  {
		       if ( Adj[i][j] == 1 )
			   {
				Neigh[i].push_back ( j );
			   }
		  }
	 }
}

////***********************************************************************************************************************************
////********************************************   FindCones Better!   ***************************************************************
////***********************************************************************************************************************************
//
//// Produces a nxnxn matrix which given three disjoint vertices is one when both: (1) three vertices are distinct and (2) the first two vertices are not connected by an edge and (3) the third is connected to each of the first two by an edge. Then it is made symmetric.
//
//void FindCones(int n)
//{     
//   for (int j=0; j<n; j++)
//      {
//              for (int k=0; k<n;k++)
//              { 
//
//                      for (int t=0; t<n;t++) 
//             {
//                  if(t==j)
//                     {
//                        Cones[j][k][t]=0;
//                        Cones[k][j][t]=0;
//                     }
//
//                  else if(t==k)
//                     {
//                        Cones[j][k][t]=0;
//                        Cones[k][j][t]=0;
//                     }
//
//                 else if(j==k)
//                      {
//                        Cones[j][k][t]=0;
//                        Cones[k][j][t]=0;
//                     }
//               
//                  else               
//                     {
//                        Cones[j][k][t]=Adj[t][j]*Adj[t][k]*(1-Adj[j][k]);
//                                               Cones[k][j][t]=Adj[t][j]*Adj[t][k]*(1-Adj[j][k]);
//                     }
//                          }
//                 }    
//         }
//}


////***********************************************************************************************************************************
////***********************   FindCones EVEN Better --- The Boolean Function Incarnation!   *******************************************
////***********************************************************************************************************************************
//
//// Instead of producing an nxnxn matrix Cones, this is a boolean function that computes what the j,k,t value of that matrix would be. Given three disjoint vertices the value of this function is one when both: (1) three vertices are distinct and (2) the first two vertices are not connected by an edge and (3) the third is connected to each of the first two by an edge.
//int Cones(int j, int k, int t)
//{
//    
//    if(t==j || t==k || j==k)
//    {
//        return 0;
//    }
//
//    
//    else
//    {
//        return Adj[t][j]*Adj[t][k]*(1-Adj[j][k]);
//    }
//   
//}


//***********************************************************************************************************************************
//***********************   FindCones Alternative --- Suspension   *******************************************
//***********************************************************************************************************************************

// Instead of producing an nxnxn matrix Cones, this is an nxn function that computes what the j,k,t value of that matrix would be. Given three disjoint vertices the value of this function is one when both: (1) three vertices are distinct and (2) the first two vertices are not connected by an edge and (3) the third is connected to each of the first two by an edge.

vector < int >
Suspension ( int j, int k, int n )
{
     vector < int >
	  suspvect;

     if ( j != k and Adj[j][k] == 0 )
	 {
//        printf("suspvect %d,%d: ", j,k);

	      for ( int t = 0; t < n; t++ )
		  {
		       if ( Adj[j][t] * Adj[k][t] == 1 )
			   {
				suspvect.push_back ( t );
//                printf("%d,",t);
			   }
		  }
//        printf("\n");
	 }

     return suspvect;

}


//***********************************************************************************************************************************
//********************************************   Connected   ************************************************************************
//***********************************************************************************************************************************

//Checks if the graph is connected
//Note: This function is adapted from a function in older software by J. Behrstock, M. Hagen, A. Sisto, available at www.wescac.net/research.html

bool
Connected ( int n )
{
     //a 1 in the vector comp corresponds to a vertex in the connected component of 0

     vector < int >
	  comp;

     comp.resize ( n );

//stores vertices in the connected component of 0 in the order they are found
     vector < int >
	  compscramble;

     comp[0] = 1;
     compscramble.push_back ( 0 );

//the iteration adds to comp and compscramble all neighbors of the vertices in the connected component of 0 
//that have been already found goes on until no more vertices have been added
     long
	  old;

     do
	 {
	      old = compscramble.size (  );
	      for ( int q = 0; q < compscramble.size (  ); q++ )
		  {
		       int
			    j = compscramble[q];

		       for ( int k = 0; k < Neigh[j].size (  ); k++ )
			   {
				if ( comp[Neigh[j][k]] == 0 )
				    {
					 comp[Neigh[j][k]] = 1;
					 compscramble.
					      push_back ( Neigh[j][k] );
				    }
			   }
		  }
	 }
     while ( old < compscramble.size (  ) );

//if the size of the connected component of 0 is n, then the graph is connected

     if ( compscramble.size (  ) == n )
	  return true;
     else
	  return false;
}




//***********************************************************************************************************************************
//********************************************   Graph Converter   ******************************************************************
//***********************************************************************************************************************************

//Converts list of TEA formated graphs to a vector of Adj matrices.

void
convertline ( string inputline, int &n, int flag_verbose, int flag_debug,
	      int flag_full )
{
     //string to be converted
     //DT2:string input = "[1,2,3,4,5,6,7] [(1,2),(2,3),(3,4),(1,4),(1,5),(3,5),(2,6),(4,6),(5,7),(6,7)]";
     //string inputline = "[1,2,3,4] [(1,2),(2,3),(3,4),(1,4)]";



     if ( flag_verbose == 1 || flag_debug == 1 )
	 {
	      //print string
	      std::cout << "\nInput string: " << inputline << "\n";
	 }

     //total string length
     int
	  inputline_length = inputline.length (  );

     //when iterating through input, this stores the current character
     string
	  current_char = "";

     //index is integer marking end of vertex description, aka when "]" first appears
     int
	  index = 0;

     //a match is found to "]"
     int
	  match = 0;

     //place holder while iterating through input
     int
	  i = 0;

     //number of vertices of graph
     int
	  vertices = 1;

     //look for "]" and count vertices
     while ( match == 0 )
	 {
	      //move to next character in string
	      current_char = inputline[i];

	      int
		   is_bracket = current_char.compare ( "]" );
	      int
		   is_comma = current_char.compare ( "," );

	      //if a comma is found before "]", this means there is a vertex
	      if ( is_comma == 0 )
		  {
		       vertices++;
		  }
	      if ( is_bracket == 0 )
		  {
		       match = 1;
		       index = i;
		  }

	      i++;
	 }

//    std::cout << "\nNumber of vertices: " << vertices;

     n = vertices;

     Adj.resize ( n );

     for ( int i = 0; i < n; i++ )
	  Adj[i].resize ( n );


     //adjacency matrix
     //  int adj[vertices][vertices];

     //make matrix all zeros
     for ( int i = 0; i < vertices; i++ )
	 {
	      for ( int j = 0; j < vertices; j++ )
		  {
		       Adj[i][j] = 0;

		  }

	 }

     //find edges
     //v1 and v2 are the vertices being described, v1_start and v2_start is their index in the input string
     //first v1 is at index+4 which is where edge list begins
     int
	  v1_start = index + 4;
     int
	  v2_start = 0;
     string
	  v1;
     string
	  v2;
     string
	  previous_char;

     for ( int i = ( index + 4 ); i < inputline_length; i++ )
	 {
	      current_char = inputline[i];
	      previous_char = inputline[i - 1];

	      int
		   is_comma = current_char.compare ( "," );

	      int
		   is_closed_parenthesis = current_char.compare ( ")" );

	      int
		   previous_closed_parenthesis =
		   previous_char.compare ( ")" );

	      //if a comma is found and no parenthesis in previous character, this signifies the start of v2 and end of v1
	      if ( is_comma == 0 && previous_closed_parenthesis != 0 )
		  {

		       int
			    v1_length = i - v1_start;

		       v1 = inputline.substr ( v1_start, v1_length );

		       v2_start = i + 1;
		  }

	      //if a closed parenthesis is found, this signifies the end of v2 and start of a new v1
	      if ( is_closed_parenthesis == 0 )
		  {

		       int
			    v2_length = i - v2_start;

		       v2 = inputline.substr ( v2_start, v2_length );

		       //update adjacency matrix
		       int
			    v1_as_int = atoi ( v1.c_str (  ) );
		       int
			    v2_as_int = atoi ( v2.c_str (  ) );

		       Adj[v1_as_int - 1][v2_as_int - 1] = 1;
		       Adj[v2_as_int - 1][v1_as_int - 1] = 1;

		       //new v1_start
		       v1_start = i + 3;
		  }



	 }


     // This loop prints the Adj matrix (ie the random graph).
     if ( flag_debug == 1 )
	 {
	      for ( int i = 0; i < n; i++ )
		   for ( int j = 0; j < n; j++ )
			printf ( "Adj[%d][%d] = %d\n", i, j, Adj[i][j] );
	 }


     //print adjacency matrix

//    std::cout << "\nAdjacency Matrix: ";
//    for(int i = 0; i < vertices; i++){
//        std::cout << "\n";
//        for(int j = 0; j < vertices; j++){
//            std::cout << Adj[i][j] << " ";
//            
//        }
//        
//    }

     //Output TEA formatted graphs:
     if ( flag_debug == 1 || flag_verbose == 1 )
	 {
	      printf ( "[0" );
	      for ( int i = 1; i < n; i++ )
		   printf ( ",%d", i );
	      printf ( "] [" );
	      for ( int i = 0; i < n; i++ )
		   for ( int j = i + 1; j < n; j++ )
			if ( Adj[i][j] == 1 )
			    {
				 printf ( "(%d,%d),", i, j );
			    }
	      printf ( "\b]\n" );
	 }


}


//***********************************************************************************************************************************
//********************************************   Clique   ***************************************************************************
//***********************************************************************************************************************************

//Connected to make sure that the core is not a clique 0=non-clique 1=clique

bool
Clique ( int j, int k, int n, vector < int >suspvect )
{
     for ( int c = 0; c < suspvect.size (  ); c++ )
	 {
	      for ( int d = c + 1; d < suspvect.size (  ); d++ )
		  {
		       if ( Adj[suspvect[c]][suspvect[d]] == 0 )
			   {
				return false;
				break;
			   }
		  }

	 }

     return true;
}
