#include <vector>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <iostream>
#include <fstream>
#include <string>
#include<sstream>
#include <cstring>
#include <numeric>

//WARNING: Unless in "obstinate mode" (see below), this program will say "not CFS" if it finds a square through every vertex, but hasn't yet found a single CFS piece, even though one might exists.

using namespace std;
using std::vector;

//This algorithm operates on a matrix Adj, which is the adjacency matrix of a graph G.
vector < vector < char > > Adj;

//The Matrix Neigh stores, for each vertex i,  all vertices adjacent to i.
vector < vector < long > > Neigh;

// ...the function Neighbors populates Neigh
vector < vector < long > > Neighbors (const vector < vector < char > >&pAdj);

//ErdosRenyi is used to produce the adjacency matrix Adj: beginning with a set of n vertices,
//ErdosRenyi decides independently with probability p whether each pair of vertices is adjacent.

void ErdosRenyi (float p, int n, float r, int flag_debug, int flag_verbose);
//void ErdosRenyi(float p, int n, float r);

//Connected checks if G is connected
bool Connected (int n);

vector < int > Suspension (const vector < vector < char > >&pAdj, int j, int k, int n);

//Clique is used to make sure that the core is not a clique 0=non-clique 1=clique
bool Clique (const vector < vector < char > >&pAdj, int j, int k, int n,
        const vector < int >&psuspvect);

//FindSquares will change the input vector by changing the entry to 1 for each vertex in a square with j and k.
int FindSquares (vector < int >&pSquareComp, const vector < vector < char > >&pAdj, int j, int k, int flag_debug, int flag_verbose);

//Grow a square to a squaregraph component
int FindCFScomponent (vector < int >&pSquareComp, const vector < vector < char > >&pAdj, vector < int >pSubgraph, int &ptested_pair, int flag_debug, int flag_verbose);

int
ObstinateFindCFScomponent (vector < int >&pSquareComp, const vector < vector < char > >&pAdj, vector < int >pSubgraph, int &ptested_pair, int &componentcount, int &totalcomponentsizes,  int &plargestcomponent, int flag_debug, int flag_verbose);

//Convert a line of input to TEA format.
void convertline (string inputline, int &n, int flag_verbose, int flag_debug, int flag_full);

//*****************************************************************************************************
//*************************************   MAIN   ******************************************************
//*****************************************************************************************************

int main (int argc, char *argv[])
{
    //The arguments entered on the command line are n, p, and tests. The option -v activates the verbose mode. The option -f activates the full output mode. The option -d activates the debug mode. The option -o activates obstinate mode, where every pair of vertices will be checked until CFS is proven.  These must be entered as the first argument.
    
    //Read command line arguments and set mode flags:
    
    int flag_verbose = 0, flag_full = 0, flag_debug = 0, flag_obstinate = 0, flag_counter = 0, flag_alpha = 0;
    
    string
    out_file;
    
    if (argv[1][0] == '-')
    {
        if (argv[1][1] == 'v')
        {
            flag_verbose = 1;
            flag_counter = 1;
        }
        if (argv[1][2] == 'v')
        {
            flag_verbose = 1;
            flag_counter = 1;
        }
        if (argv[1][3] == 'v')
        {
            flag_verbose = 1;
            flag_counter = 1;
        }
        if (argv[1][4] == 'v')
        {
            flag_verbose = 1;
            flag_counter = 1;
        }
    }
    
    if (argv[1][0] == '-')
    {
        if (argv[1][1] == 'f')
        {
            flag_full = 1;
            flag_counter = 1;
        }
        if (argv[1][2] == 'f')
        {
            flag_full = 1;
            flag_counter = 1;
        }
        if (argv[1][3] == 'f')
        {
            flag_full = 1;
            flag_counter = 1;
        }
        if (argv[1][4] == 'f')
        {
            flag_full = 1;
            flag_counter = 1;
        }
    }
    
    if (argv[1][0] == '-')
    {
        if (argv[1][1] == 'd')
        {
            flag_debug = 1;
            flag_counter = 1;
        }
        if (argv[1][2] == 'd')
        {
            flag_debug = 1;
            flag_counter = 1;
        }
        if (argv[1][3] == 'd')
        {
            flag_debug = 1;
            flag_counter = 1;
        }
        if (argv[1][4] == 'd')
        {
            flag_debug = 1;
            flag_counter = 1;
        }
    }
    
    int flag_teaformatted = 0;
    
    if (argv[1][0] == '-')
    {
        if (argv[1][1] == 't')
        {
            flag_teaformatted = 1;
            flag_counter = 1;
        }
        if (argv[1][2] == 't')
        {
            flag_teaformatted = 1;
            flag_counter = 1;
        }
        if (argv[1][3] == 't')
        {
            flag_teaformatted = 1;
            flag_counter = 1;
        }
        if (argv[1][4] == 't')
        {
            flag_teaformatted = 1;
            flag_counter = 1;
        }
    }
    
    if (argv[1][0] == '-')
    {
        if (argv[1][1] == 'o')
        {
            flag_obstinate = 1;
            flag_verbose = 1;
            flag_counter = 1;
        }
        if (argv[1][2] == 'o')
        {
            flag_obstinate = 1;
            flag_verbose = 1;
            flag_counter = 1;
        }
        if (argv[1][3] == 'o')
        {
            flag_obstinate = 1;
            flag_verbose = 1;
            flag_counter = 1;
        }
        if (argv[1][4] == 'o')
        {
            flag_obstinate = 1;
            flag_verbose = 1;
            flag_counter = 1;
        }
    }
    
    if (flag_obstinate == 1 && flag_full == 1 && argv[1][1] != 'v'
        && argv[1][2] != 'v' && argv[1][3] != 'v' && argv[1][4] != 'v')
    {
        flag_verbose = 0;
    }
    
    if (argv[1][0] == '-')
    {
        if (argv[1][1] == 'a')
        {
            flag_alpha = 1;
            flag_counter = 1;
        }
        if (argv[1][2] == 'a')
        {
            flag_alpha = 1;
            flag_counter = 1;
        }
        if (argv[1][3] == 'a')
        {
            flag_alpha = 1;
            flag_counter = 1;
        }
        if (argv[1][4] == 'a')
        {
            flag_alpha = 1;
            flag_counter = 1;
        }
    }
    
    //p is the density of edges, n is the number of vertices, and tests is the number of tests to run:
    
    float p = 1;
    
    int n = 1;
    
    int tests = 1;
    //-----------------------------------------------------------------------------------------------------------------------------------------------------
    
    //Set p and n if not using a specific graph in TEA mode:
    
    if (flag_teaformatted == 0)
    {
        p = atof (argv[flag_counter + 2]);
        
        n = atoi (argv[flag_counter + 1]), tests =
        atoi (argv[flag_counter + 3]);
    }
    
    //If running in alpha mode, change the value of p:
    
    float alpha = p;
    
    if (flag_alpha == 1)
    {
        p = alpha * pow (n, -.5);
    }
    
    if (flag_alpha != 1)
    {
        alpha = p/pow (n, -.5);
    }
    
    string inputfile;
    
    string inputfilename;
    
    string inputline;
    
    // cfs_total will report the number of tests which find a CFS graph
    int cfs_total = 0;
    
    // prop_cfs will tell the percent of tests which are CFS
    float prop_cfs = 0;
    
    // r will be the random seed to produce a random graph.
    float r;
    
    srand (time (NULL));
    
    //If running in TEA mode, read how many graphs graphs; if in full mode, report on it:
    
    if (flag_teaformatted == 1)
    {
        int numLines = 0;
        ifstream inputfilename (argv[2]);
        string unused;
        while (getline (inputfilename, unused))
        ++numLines;
        
        tests = numLines;
        
        if (flag_full != 1)
        {
            printf ("Running %d graphs from file.\n", tests);
        }
    }
    
    // This is the core of the algorithm. It will be run "test" time, as measured by the counter curr_test:
    
    for (int curr_test = 0; curr_test < tests; curr_test++)
    {
        int increased = 0;
        int total_norm = 0;
        
        //First we create our graph:
        
        //Random graph:
        if (flag_teaformatted == 0)
        {
            //first we make the relevant vectors and resize them to n entries:
            
            Adj.resize (n);
            
            for (int i = 0; i < n; i++)
            Adj[i].resize (n);
            
            //create graph
            
            ErdosRenyi (p, n, r, flag_debug, flag_verbose);
        }
        
        //User input graphs from file:
        if (flag_teaformatted == 1)
        {
            string line;
            ifstream inputfilename (argv[2]);
            if (inputfilename.is_open ())
            {
                for (int lineno = 0;
                     getline (inputfilename, line)
                     && lineno < tests; lineno++)
                {
                    if (lineno == curr_test)
                    inputline = line;
                }
                
                inputfilename.close ();
            }
            
            convertline (inputline, n, flag_verbose, flag_debug,
                         flag_full);
            
        }
        
        // vectorSquareComp will keep track of the vectors in any of the CFS components found thus far:
        // The first entry is the component number the second are the entries in that component.
        vector < vector < int > > vectorSquareComp;
        vectorSquareComp.resize (1);
        vectorSquareComp[0].resize (n);
        
        //We now populate Neigh...
        
        Neigh = Neighbors (Adj);
        
        //...and only proceed further if the graph is connected.
        
        if (Connected (n) == true)
        {
            int tested_pair = 0;
            int componentcount = 0;
            
            //norm will keep track of which vertices have been assigned to a CFS component.
            int norm = 0;
            
            //create a subgraph of vertices which we haven't yet put into any CFS component. Start with the subgraph being all n vertices of the graph. This will be complementary to the SquareComp vector.
            vector < vector < int > > Subgraph;
            Subgraph.resize (1);
            Subgraph[0].resize (n);
            for (int i = 0; i < n; i++)
            {
                Subgraph[0][i] = 1;
            }
            
            //now search for squares and when one is found grow it to the largest CFS subgraph containing it. The output measures how many vertices are in this CFS component.
            norm = FindCFScomponent (vectorSquareComp[0], Adj, Subgraph[0], tested_pair, flag_debug, flag_verbose);
            
            int largestcomponent = norm;
            
            // when norm=n we have accounted for all vertices.
            if (norm == n)
            {
                cfs_total++;  //The graph is CFS
                increased = 1;
                
                if (flag_verbose == 1 || flag_debug == 1)
                {
                    printf ("This graph is CFS\n");
                }
            }
            
            if (norm < n && flag_verbose == 1)
            {
                printf
                ("Component number %d has size %d\n",
                 componentcount + 1, norm);
            }
            
            int totalcomponentsizes = norm;
            
            ///main part of algorithm
            if (flag_obstinate == 0)
            {
                while (norm < n)
                {
                    //For each vertex, add it to the test pool if it isn't already supported on a component of the squaregraph
                    for (int i = 0; i < n; i++)
                    {
                        Subgraph[0][i] = 1 - vectorSquareComp [0][i];
                        
                    }
                    
                    //Compute new component and its size
                    int temp_componentsize = FindCFScomponent (vectorSquareComp[0], Adj, Subgraph[0], tested_pair, flag_debug, flag_verbose);
                    
                    if (temp_componentsize == 0)
                    {
                        break;
                    }
                    
                    //Update number of components
                    componentcount = componentcount + 1;
                    
                    //Update size of largest component
                    if (temp_componentsize > largestcomponent)
                    {
                        largestcomponent = temp_componentsize;
                    }
                    
                    if (flag_verbose == 1 || flag_debug == 1)
                    {
                        printf
                        ("Component number %d has size %d\n", componentcount + 1,temp_componentsize);
                    }
                    
                    //If the new component supports everything, we're CFS and done
                    if (temp_componentsize == n)
                    {
                        cfs_total++;  //Update CFS count
                        increased = 1;
                        if (flag_verbose == 1 || flag_debug == 1)
                        {
                            printf
                            ("This graph is CFS\n");
                        }
                    }
                    
                    //Total component sizes (for computing average component size)
                    totalcomponentsizes = totalcomponentsizes + temp_componentsize;
                    
                    //Compute total number of supported vertices
                    norm = std::accumulate (vectorSquareComp[0].begin(), vectorSquareComp[0].end(), 0);
                    
                    if (flag_debug == 1)
                    {
                        printf("Run %d has norm %d\n", componentcount +1, norm);
                    }
                }
            }
            ///end main part of algorithm
            
            //obstinate algorithm: same as above, except use the obstinate version of FindCFScomponent, which does all components in one shot, instead of one at a time, and continues until all components have been tested (or until CFS is verified)
            if (flag_obstinate == 1 && norm < n)
            {
                increased = ObstinateFindCFScomponent (vectorSquareComp[0], Adj, Subgraph[0], tested_pair, componentcount, totalcomponentsizes, largestcomponent, flag_debug, flag_verbose);
                
                cfs_total = cfs_total + increased;
            }
            ///end obstinate algorithm
            
            //Report non-CFS in verbose/debug modes:
            if (flag_verbose == 1 || flag_debug == 1)
            {
                if (increased == 0)
                {
                    printf("This graph is not CFS\n");
                    
                }
            }
            
            //Compute average component size:
            float avgcomponentsize = static_cast < float >(totalcomponentsizes) / (componentcount + 1);
            
            //report on individual tests.
            if (flag_full == 1)
            {
                
                //Output (Vertices, Density, alpha, Whether CFS, # tested pairs before components found,  numbers of components, average component size, size of largest component):
                
                cout << n << ", " << p << ", " << alpha << ", " <<  increased << ", " << tested_pair <<  ", " << componentcount + 1 << ", " << avgcomponentsize << ", " << largestcomponent << "\n";
            }
        }
        
        else			  // HAVE IT DO DIFFERENT THINGS IN THE DISCONNECTED CASE ACCORDING TO FLAGS
        {
            //printf("Disconnected, hence not thick.");
            
            if (flag_verbose == 1)
            {
                printf ("This graph is disconnected\n");
            }
            
            if (flag_full == 1)	//report on individual tests:
            {
                //Output (Vertices, Density, alpha, Whether CFS, # good pairs before positive ID):
                
                cout << n << ", " << p << ", " << alpha << ", " << "0" << ", " << "Disconnected." << "\n";

            }
        }
    }
    
    //Info to report:
    
    if (flag_full != 1)
    {
        prop_cfs = 100 * static_cast < float >(cfs_total) / tests;
        printf ("p=%f, alpha=%f, n=%d, trials= %d, #CFS= %d, PercentCFS= %f\n", p, alpha, n, tests, cfs_total, prop_cfs);
    }
    
}

//***********************************************************************************************************************************
//********************************************   ErdosRenyi   ***********************************************************************
//***********************************************************************************************************************************

void ErdosRenyi (float p, int n, float r, int flag_debug, int flag_verbose)
{
    //This function creates our random graph my creating a random symmetric matrix. It sets Adj[i][j]=Adj[j][i] to 0 or 1 according to whether a random number r in [0,1] exceeds p.
    
    for (int i = 0; i < n; i++)
    {
        for (int j = i + 1; j < n; j++)
        {
            r = (float) rand () / (float) RAND_MAX;
            if (r < p)
            {
                Adj[i][j] = 1;
                Adj[j][i] = 1;
            }
            
            else
            {
                Adj[i][j] = 0;
                Adj[j][i] = 0;
            }
        }
    }
    
    //Tea formatted graphs (importable):
    if (flag_verbose == 1)
    {
        printf ("Graph: [1");
        for (int i = 1; i < n; i++)
        printf (",%d", i + 1);
        printf ("] [");
        for (int i = 0; i < n; i++)
        for (int j = i + 1; j < n; j++)
        if (Adj[i][j] == 1)
        {
            printf ("(%d,%d),", i + 1, j + 1);
        }
        printf ("\b]\n");
    }
}

//***********************************************************************************************************************************
//********************************************   Connected   ************************************************************************
//***********************************************************************************************************************************

//Checks if the graph is connected

bool Connected (int n)
{
    //a 1 in the vector comp corresponds to a vertex in the connected component of 0
    
    vector<int> comp;
    comp.clear();
    comp.resize(n);
    
    //stores vertices in the connected component of 0 in the order they are found
    vector<int> compscramble;
    compscramble.clear();
    
    comp[0] = 1;
    compscramble.push_back (0);
    
    //the iteration adds to comp and compscramble all neighbors of the vertices in the connected component of 0
    //that have been already found goes on until no more vertices have been added
    long
    old;
    
    do
    {
        old = compscramble.size ();
        for (int q = 0; q < compscramble.size (); q++)
        {
            int
            j = compscramble[q];
            for (int k = 0; k < Neigh[j].size (); k++)
            {
                if (comp[Neigh[j][k]] == 0)
                {
                    comp[Neigh[j][k]] = 1;
                    compscramble.
                    push_back (Neigh[j][k]);
                }
            }
        }
    }
    while (old < compscramble.size ());
    
    //if the size of the connected component of 0 is n, then the graph is connected
    
    if (compscramble.size () == n)
    return true;
    else
    return false;
}

//***********************************************************************************************************************************
//********************************************   Graph Converter   ******************************************************************
//***********************************************************************************************************************************

//Converts list of TEA formated graphs to a vector of Adj matrices.
void convertline (string inputline, int &n, int flag_verbose, int flag_debug,
             int flag_full)
{
    if (flag_debug == 1)
    {
        //print string
        std::cout << "\nInput string: " << inputline << "\n";
    }
    
    //total string length
    int inputline_length = inputline.length ();
    
    //when iterating through input, this stores the current character
    string current_char = "";
    
    //index is integer marking end of vertex description, aka when "]" first appears
    int index = 0;
    
    //a match is found to "]"
    int match = 0;
    
    //place holder while iterating through input
    int i = 0;
    
    //number of vertices of graph
    int vertices = 1;
    
    //look for "]" and count vertices
    while (match == 0)
    {
        //move to next character in string
        current_char = inputline[i];
        
        int is_bracket = current_char.compare ("]");
        int is_comma = current_char.compare (",");
        
        //if a comma is found before "]", this means there is a vertex
        if (is_comma == 0)
        {
            vertices++;
        }
        if (is_bracket == 0)
        {
            match = 1;
            index = i;
        }
        
        i++;
    }
    
    n = vertices;
    
    Adj.resize (n);
    
    for (int i = 0; i < n; i++)
    Adj[i].resize (n);
    
    //make matrix all zeros
    for (int i = 0; i < vertices; i++)
    {
        for (int j = 0; j < vertices; j++)
        {
            Adj[i][j] = 0;
        }
    }
    
    //find edges
    //v1 and v2 are the vertices being described, v1_start and v2_start is their index in the input string
    //first v1 is at index+4 which is where edge list begins
    int v1_start = index + 4;
    int v2_start = 0;
    string v1;
    string v2;
    string previous_char;
    
    for (int i = (index + 4); i < inputline_length; i++)
    {
        current_char = inputline[i];
        previous_char = inputline[i - 1];
        
        int is_comma = current_char.compare (",");
        
        int is_closed_parenthesis = current_char.compare (")");
        
        int previous_closed_parenthesis = previous_char.compare (")");
        
        //if a comma is found and no parenthesis in previous character, this signifies the start of v2 and end of v1
        if (is_comma == 0 && previous_closed_parenthesis != 0)
        {
            
            int v1_length = i - v1_start;
            
            v1 = inputline.substr (v1_start, v1_length);
            
            v2_start = i + 1;
        }
        
        //if a closed parenthesis is found, this signifies the end of v2 and start of a new v1
        if (is_closed_parenthesis == 0)
        {
            
            int v2_length = i - v2_start;
            
            v2 = inputline.substr (v2_start, v2_length);
            
            //update adjacency matrix
            int v1_as_int = atoi (v1.c_str ());
            int v2_as_int = atoi (v2.c_str ());
            
            Adj[v1_as_int - 1][v2_as_int - 1] = 1;
            Adj[v2_as_int - 1][v1_as_int - 1] = 1;
            
            //new v1_start
            v1_start = i + 3;
        }
        
    }
    
    //Output TEA formatted graphs --- Actual graph:
    if (flag_debug == 1 || flag_verbose == 1)
    {
        printf ("Graph : [0");
        for (int i = 1; i < n; i++)
        printf (",%d", i);
        printf ("] [");
        for (int i = 0; i < n; i++)
        for (int j = i + 1; j < n; j++)
        if (Adj[i][j] == 1)
        {
            printf ("(%d,%d),", i, j);
        }
        printf ("\b]\n");
    }
}

//***********************************************************************************************************************************
//*********************************************   Neighbors   ***********************************************************************
//***********************************************************************************************************************************

//Produces an matrix where Neigh[i] contains the list of vertices adjacent to vertex i

vector < vector < long > > Neighbors (const vector < vector < char > >&pAdj)
{
    vector < vector < long > > Neigh;
    
    int n = pAdj[0].size ();
    
    Neigh.resize (n);
    
    for (unsigned i = 0; i < Neigh.size (); i++)
    {
        Neigh[i].clear ();
    }
    
    for (int i = 0; i < n; i++)
    {
        for (int j = 0; j < n; j++)
        {
            if (pAdj[i][j] == 1)
            {
                Neigh[i].push_back (j);
            }
        }
    }
    
    return Neigh;
    
}

//***********************************************************************************************************************************
//*************************************************   Suspension   ******************************************************************
//***********************************************************************************************************************************

// Given three disjoint vertices the value of this function is one when both: (1) three vertices are distinct and (2) the first two vertices are not connected by an edge and (3) the third is connected to each of the first two by an edge.

vector < int > Suspension (const vector < vector < char > >&pAdj, int j, int k, int n)
{
    vector < int > suspvect;
    suspvect.reserve (n);
    
    if (j != k && pAdj[j][k] == 0)
    {
        for (int t = 0; t < n; t++)
        {
            if (pAdj[j][t] == 1 && pAdj[k][t] == 1)
            {
                suspvect.push_back (t);
            }
        }
    }
    return suspvect;
}

//***********************************************************************************************************************************
//********************************************   Clique   ***************************************************************************
//***********************************************************************************************************************************

//Check that the core is not a clique 0=non-clique 1=clique

bool Clique (const vector < vector < char > >&pAdj, int j, int k, int n, const vector < int >&psuspvect)
{
    for (int c = 0; c < psuspvect.size (); c++)
    {
        for (int d = c + 1; d < psuspvect.size (); d++)
        {
            if (pAdj[psuspvect[c]][psuspvect[d]] == 0)
            {
                return false;
                break;
            }
        }
    }
    
    return true;
}

//***********************************************************************************************************************************
//********************************************   FindSquares          ***************************************************************
//***********************************************************************************************************************************

// This finds all squares adjacent to j and k, a pair of distinct non-adjacent vertices. This updates the value in the first vector which records which squares are in the cfs component of these vertices. The value returned is the number of new vertices added to that vector.

int FindSquares (vector < int >&pSquareComp, const vector < vector < char > >&pAdj, int j, int k, int flag_debug, int flag_verbose)
{//Rule out the case where (j,k) is an edge:
    if (j == k || pAdj[j][k] == 1)
    {
        if (flag_debug == 1)
        {
            printf
            ("Running FindSquares(%d,%d): these vertices are adjacent.\n",
             j, k);
        }
        return 0;
    }
    //How many new vertices added to the support of the CFS component in question
    int temp_norm = 0;
    
    //Size of the squaregraph component in question
    int  n = pSquareComp.size ();
    
    if (flag_debug == 1)
    {
        printf ("Running FindSquares(%d,%d):\n", j, k);
    }
    
    //Build the j,k block
    vector < int >  suspvect = Suspension (pAdj, j, k, n);
    
    //Test if the block is a clique; if not...
    if (Clique (pAdj, j, k, n, suspvect) == false)
    {
        //...add j and increment the number of vertices
        if (pSquareComp[j] == 0)
        {
            pSquareComp[j] = 1;
            temp_norm = temp_norm + 1;
            
            if (flag_debug == 1)
            {
                printf ("Added vertex %d \n", j);
            }
        }
        //same as above, but for k
        if (pSquareComp[k] == 0)
        {
            pSquareComp[k] = 1;
            temp_norm = temp_norm + 1;
            
            if (flag_debug == 1)
            {
                printf ("Added vertex %d \n", k);
            }
        }
        
        for (int s = 0; s < suspvect.size (); s++)
        {
            //Report all values if in debug mode
            if (flag_debug == 1)
            {
                printf ("supvect(%d,%d)[%d]= %d \n", j, k,
                        s, suspvect[s]);
                printf ("pSquareComp[%d]=%d\n",
                        suspvect[s],
                        pSquareComp[suspvect[s]]);
                printf ("Checking vertex %d \n",
                        suspvect[s]);
                
            }
            //same as above, but for core vertices
            if (pSquareComp[suspvect[s]] == 0)
            {
                pSquareComp[suspvect[s]] = 1;
                temp_norm = temp_norm + 1;
                
                if (flag_debug == 1)
                {
                    printf ("Added vertex %d \n",
                            suspvect[s]);
                }
            }
        }
    }
    return temp_norm;
}

//***********************************************************************************************************************************
//********************************************   FindCFScomponent               *****************************************************
//***********************************************************************************************************************************

//Build a component of the squaregraph and return the number of vertices it supports.

int FindCFScomponent(vector<int>& pSquareComp, const vector<vector<char> >& pAdj, vector<int>  pSubgraph, int& ptested_pair, int flag_debug, int flag_verbose)
{
    //    int begin_temp_norm=std::accumulate(pSquareComp.begin(),pSquareComp.end(),0);
    int increased=0;
    vector<int> cfs_component;
    int n= pSquareComp.size();
    cfs_component.resize(n);
    for (int s=0; s<n; s++)
    {
        cfs_component[s]=0;
    }
    
    if (flag_debug == 1)
    {
        printf ("RunningFindCFScomp\n");
        
    }
    
    //first we look at pairs of vertices j,k and make sure they are in the subgraph we are considering. If so we check to see if they are in a square, if not, we keep looking until we find a square.
    
    for (int j = 0; j < n; j++)
    {
        if (pSubgraph[j] != 1)
        {
            continue;
        }
        for (int k = 0; k < j; k++)
        {
            if (pSubgraph[k] != 1)
            {
                continue;
            }
            
            //we first throw out the case where j and k are adjacent
            if (pAdj[j][k] == 1)
            {
                if (flag_debug == 1)
                {
                    printf
                    ("No new squares: vertices are adjacent\n");
                }
                
                ptested_pair = ptested_pair + 1;
                
                continue;
            }
            
            // we now look for squares containing j,k as opposite vertices. If we find one we add the vertices to SquareComp.
            // the following part of the loop is reached only when (pAdj[j][k]==0)
            
            int value_findsquares = FindSquares (pSquareComp, pAdj, j, k, flag_debug, flag_verbose);
            
            ptested_pair = ptested_pair + 1;
            
            if (value_findsquares > 0)
            {
                increased = 1;
                cfs_component[j] = 1;
                cfs_component[k] = 1;
                
                vector < int > suspensionjk = Suspension (Adj, j, k, n);
                
                for (int s = 0; s < suspensionjk.size (); s++)
                {
                    cfs_component[suspensionjk[s]] = 1;
                }
                
                if (flag_debug == 1)
                {
                    printf
                    ("Vertices (%d,%d) lie on a square; now we'll search for the CFS component. \n", j, k);
                }
                
            }
            
            if (increased == 1)
            break;
            
        }
        
        if (increased == 1)
        break;
    }
    
    //if increased==0, then there are no squares left and we exit this routine.
    if (increased == 0)
    {
        return 0;
    }
    
    //otherwise, increased==1 and we found a square. So now we construct the CFS component containing it.
    
    int temp_norm = std::accumulate (cfs_component.begin (), cfs_component.end (), 0);
    
    if (temp_norm == n)
    {
        return temp_norm;
    }
    
    if (flag_debug == 1)
    {
        for (int s = 0; s < n; s++)
        {
            printf ("cfscomponent[%d]=%d\n", s, cfs_component[s]);
        }
    }
    
    int
    vertexadded = 0;
    
    do
    {
        vertexadded = 0;
        
        int temp_norm_before = std::accumulate (cfs_component.begin (), cfs_component.end (), 0);
        
        for (int s = 1; s < n; s++)
        {
            if (cfs_component[s] != 1)
            {
                continue;
            }
            for (int t = 0; t < s; t++)
            {
                if (cfs_component[t] != 1)
                {
                    continue;
                }
                
                FindSquares (cfs_component, pAdj, s, t, flag_debug, flag_verbose);
            }
        }
        
        int temp_norm_after = std::accumulate (cfs_component.begin (), cfs_component.end (), 0);
        int new_vertices = temp_norm_after - temp_norm_before; vertexadded = vertexadded + new_vertices;
        
        for (int s = 0; s < cfs_component.size (); s++)
        {
            if (cfs_component[s] == 1)
            {
                pSquareComp[s] = cfs_component[s];
            }
        }
    }
    while (vertexadded > 0);
    
    temp_norm = std::accumulate (cfs_component.begin (), cfs_component.end (), 0);
    
    if (flag_debug == 1)
    {
        for (int s = 0; s < n; s++)
        {
            printf ("cfscomponent[%d]=%d\n", s,
                    cfs_component[s]);
        }
        printf ("tempnorm=%d\n", temp_norm);
    }
    
    return temp_norm;
}

//***********************************************************************************************************************************
//********************************************   ObstinateFindCFScomponent     ******************************************************
//***********************************************************************************************************************************

//


int ObstinateFindCFScomponent(vector<int>& pSquareComp,const vector<vector<char> > & pAdj, vector<int>  pSubgraph, int& ptested_pair, int& componentcount , int& totalcomponentsizes, int& largestcomponent, int flag_debug, int flag_verbose)
{
    //    int begin_temp_norm=std::accumulate(pSquareComp.begin(),pSquareComp.end(),0);
    int increased=0;
    int n= pSquareComp.size();
    vector<int> cfs_component;
    cfs_component.resize(n);
    
    vector<vector<int> > vector_cfs_components;
    
    
    
    if (flag_debug == 1)
    {
        printf ("Running ObstinateFindCFScomp\n");
        
    }
    
    //first we look at pairs of vertices j,k and make sure they are in the subgraph we are considering. If so we check to see if they are in a square, if not, we keep looking until we find a square.
    
    for (int j = 0; j < n; j++)
    {
        for (int k = 0; k < j; k++)
        {
            //we first throw out the case where j and k are adjacent
            if (pAdj[j][k] == 1 && pSubgraph[j] == 1 && pSubgraph[k] == 1)
            {
                if (flag_debug == 1)
                {
                    printf
                    ("No new squares: vertices are adjacent\n");
                }
                
                ptested_pair = ptested_pair + 1;
                
            }
            
            // we now look for squares containing j,k as opposite vertices. If we find one we add the vertices to SquareComp.
            if (pAdj[j][k] == 0 && pSubgraph[j] == 1 && pSubgraph[k] == 1)
            {
                for (int s = 0; s < n; s++)
                {
                    pSquareComp[s] = 0;
                }
                
                int newcomponent = 1;
                int value_findsquares =  FindSquares (pSquareComp, pAdj, j, k, flag_debug, flag_verbose);
                
                ptested_pair = ptested_pair + 1;
                
                if (value_findsquares > 0)
                {
                    
                    for (int s = 0; s < n; s++)
                    {
                        cfs_component[s] = 0;
                    }
                    
                    cfs_component[j] = 1;
                    cfs_component[k] = 1;
                    
                    for (int s = 0;
                         s < Suspension (Adj, j, k, n).size ();
                         s++)
                    {
                        cfs_component [Suspension (Adj, j, k, n)[s]] = 1;
                    }
                    
                    if (flag_debug == 1)
                    {
                        printf("Vertices (%d,%d) lie on a square; now we'll search for the CFS component. \n", j, k);
                    }
                    
                    //THIS IS WHERE THE OBSTINATE VERSION BEGINS TO DIFFER FROM THE STANDARD ONE:
                    
                    int temp_norm =std::accumulate (cfs_component.begin (),cfs_component.end (), 0);
                    
                    if (flag_debug == 1)
                    {
                        for (int s = 0; s < n; s++)
                        {
                            printf("cfscomponent[%d]=%d\n", s,cfs_component[s]);
                        }
                    }
                    
                    if (temp_norm == n)
                    {
                        increased = 1;
                        largestcomponent = n;
                        return increased;
                    }
                    
                    int
                    vertexadded = 0;
                    
                    do
                    {
                        
                        vertexadded=0;
                        
                        for (int s=1; s<n; s++)
                        {
                            for (int t=0; t<s; t++)
                            {
                                if (cfs_component[s]==1 && cfs_component[t]==1)
                                {
                                    int temp_norm_before=std::accumulate(cfs_component.begin(),cfs_component.end(),0);
                                    int new_findsquares = FindSquares(cfs_component, pAdj,  s,  t,  flag_debug,  flag_verbose);
                                    int temp_norm_after=std::accumulate(cfs_component.begin(),cfs_component.end(),0);
                                    int new_vertices =temp_norm_after-temp_norm_before;
                                    vertexadded=vertexadded+new_vertices;
                                    
                                    if (temp_norm_after==n)
                                    {
                                        increased=1;
                                        componentcount=componentcount+1;
                                        if (flag_verbose==1 || flag_debug==1)
                                        {
                                            printf("Component number %d shows this graph is CFS\n", componentcount+1);
                                        }
                                        totalcomponentsizes=totalcomponentsizes+temp_norm_after;
                                        return increased;
                                    }
                                    
                                }
                                
                                
                                
                            }
                        }
                        
                    }
                    while (vertexadded > 0);
                    
                    
                    temp_norm=std::accumulate(cfs_component.begin(),cfs_component.end(),0);
                    
                    if (temp_norm>largestcomponent)
                    {
                        largestcomponent=temp_norm;
                    }
                    
                    
                    for (int s=0; s<vector_cfs_components.size();s++)
                    {
                        if (cfs_component==vector_cfs_components[s])
                        {
                            newcomponent=0;
                            temp_norm=0;
                            
                            if (flag_debug==1 || flag_verbose==1)
                            {
                                printf("Component searched was previously found as Component %d\n", s+2);
                            }
                            
                            
                        }
                    }
                    
                    
                    if (newcomponent==1)
                    {
                        vector_cfs_components.push_back(cfs_component);
                        componentcount=componentcount+1;
                        
                    }
                    
                    
                    
                    if (temp_norm==n)
                    {
                        
                        if (flag_verbose==1 || flag_debug==1)
                        {
                            printf("Component number %d shows this graph is CFS\n", componentcount+1);
                        }
                        increased=1;
                        return increased;
                    }
                    
                    
                    if (temp_norm<n && newcomponent==1)
                    {
                        if (flag_verbose==1 || flag_debug==1) {
                            printf("Component number %d has size %d\n", componentcount+1, temp_norm);
                        }
                        
                    }
                    
                    
                    
                    if (flag_debug==1)
                    {
                        for (int s=0; s<n; s++)
                        {
                            printf("cfscomponent[%d]=%d\n",s,cfs_component[s]);
                        }
                        printf("tempnorm=%d\n",temp_norm);
                        
                    }
                    
                    
                    
                    totalcomponentsizes=totalcomponentsizes+temp_norm;
                    
                    
                }
                
            }
            
            if (increased == 1)
            break;
            
        }
        
        if (increased == 1)
        break;
    }
    
    //if increased==0, then this is not CFS.
    if (increased == 0)
    {
        return 0;
    }
    
    return increased;
}
